%%% Script tutorial for Compressed Sensing
%
% l1-signal processing
% adapted from scripts of Romberg & Cands
%
% pb 11/2007

cd /Users/pborgnat/Arborescence/Autre_Travail/CS_from2007/l1magic
addpath(genpath('/Users/pborgnat/Arborescence/Autre_Travail/CS_from2007/'))
%%

% put key subdirectories in path if not already there
path(path, './Optimization');
path(path, './Measurements');
path(path, './Data');

disp('%%%%%% Begin tutorial on Compressed Sensing %%%%%%');

%% 1-a %% Spiky Signal - random measurement
disp('%%1%% Recovery of Sparse Spiky signal from a small number of measurement %%%%');

%%% From l1eq_example.m

% load random states for repeatable experiments
load RandomStates
rand('state', rand_state);
randn('state', randn_state);

% signal length
N = 512;%8192;
% number of spikes in the signal
T = 10;%100;
% number of observations to make
K = 128;%1024;

% random +/- 1 signal
x = zeros(N,1);
q = randperm(N);
x(q(1:T)) = sign(randn(T,1));

% measurement matrix
disp('Creating measurement matrix...');
A = randn(K,N);
A = orth(A')';
disp('Done.');
	
% observations
y = A*x;

% initial guess = min energy
x0 = A'*y;

%% Figures successives
disp('---- Step by step illustration on l1-recovery');
disp('---- l2 recovery from few measurements ----');
figure(1); clf;
plot(x,'k-','linewidth',2); hold on ; plot(q(1:T),x(q(1:T)),'*k'); axis([0 520, -1.2 1.2]);
title('Signal original K-Parcimonieux','FontSize',20);
xlabel('Espace / Temps','FontSize',20);
disp('  -- paused'); pause;

figure(2); clf;
imagesc(A'); axis('square');
title('Matrice de mesure aleatoire (gaussienne)','FontSize',20);
disp('  -- paused'); pause;

figure(3); clf;
plot(y,'k.-'); hold on ; axis([0 120, -1.2 1.2]); 
title('Signal mesure de DIMENSION reduite','FontSize',20);
xlabel('Axe variable mesuree','FontSize',20);
disp('  -- paused'); pause;

figure(4); clf;
plot(x,'k-'); hold on ; plot(q(1:T),x(q(1:T)),'*k'); axis([0 520, -1.2 1.2]);
plot(x0,'b.-'); plot(q(1:T),x0(q(1:T)),'sb');
title('Reconstruction en Optimisation l2','FontSize',20);
xlabel('Espace / Temps','FontSize',20);
disp('  -- paused'); pause;

disp('---- Next Step : l1 recovery ----');

%% 1-b %% Exact recovery

% load random states for repeatable experiments
load RandomStates
rand('state', rand_state);
randn('state', randn_state);

l1eq_example;

figure(3); clf;
plot(x,'k-'); hold on ; plot(q(1:T),x(q(1:T)),'*k'); axis([0 520, -1.2 1.2]);
plot(xp,'b.-'); plot(q(1:T),xp(q(1:T)),'sb');
title('Reconstruction en Optimisation l1 avec galit','FontSize',20);
xlabel('Espace / Temps','FontSize',20);

disp('  -- paused'); pause;


%% 1-c %% Quadratic Constraint

disp('---- Same measurement setting with added noise (variance 0.005)');
disp('  -- paused'); pause;

l1qc_example;

%% Figures successives
figure(1); clf;
plot(x,'k-'); hold on ; plot(q(1:T),x(q(1:T)),'*k'); axis([0 520, -1.2 1.2]);
title('Signal original SPARSE','FontSize',20);
xlabel('Espace / Temps','FontSize',20);
disp('  -- paused'); pause;

figure(2); clf;
imagesc(A'); axis('square');
title('Matrice de mesure aleatoire (gaussienne)','FontSize',20);
disp('  -- paused'); pause;

figure(3); clf;
plot(y,'k.-'); hold on ; plot(A*x,'b-'); axis([0 120, -1.2 1.2]); 
title('Signal +BRUIT mesure de DIMENSION reduite','FontSize',20);
xlabel('Axe variable mesuree','FontSize',20);
disp('  -- paused'); pause;

figure(4); clf;
plot(x,'k-'); hold on ; plot(q(1:T),x(q(1:T)),'*k'); axis([0 520, -1.2 1.2]);
plot(x0,'b.-'); plot(q(1:T),x0(q(1:T)),'sb');
title('Reconstruction en Optimisation l2','FontSize',20);
xlabel('Espace / Temps','FontSize',20);
disp('  -- paused'); pause;

figure(3); clf;
plot(x,'k-'); hold on ; plot(q(1:T),x(q(1:T)),'*k'); axis([0 520, -1.2 1.2]);
plot(xp,'b.-'); plot(q(1:T),xp(q(1:T)),'sb');
title('Reconstruction en Optimisation l1 avec QC','FontSize',20);
xlabel('Espace / Temps','FontSize',20);
disp('  -- paused'); pause;

%% 2-a %% Image processing from truncated/small number of Fourier measurements

disp('%%1%% Recovery of (piecewise smooth image) from a small number of Fourier measurement %%%%');

disp('---- Measurement setting ----');
for jj=1:4; figure(jj); clf; end;
disp('  -- paused'); pause;

%%% tveq_phantom_example.m

% Phantom 
n = 64*2;
N = n*n;
X = phantom(n);
x = X(:);

% number of radial lines in the Fourier domain
L = 22;

% Fourier samples we are given
[M,Mh,mh,mhi] = LineMask(L,n);
OMEGA = mhi;
A = @(z) A_fhp(z, OMEGA);
At = @(z) At_fhp(z, OMEGA, n);

% measurements
y = A(x);

% min l2 reconstruction (backprojection)
xbp = At(y);
Xbp = reshape(xbp,n,n);

figure(1); clf;
imagesc(X); colormap(gray); axis('square')
title('Image originelle (lisse par morceaux)','FontSize',20);
disp('  -- paused'); pause;

figure(2); clf;
imagesc(fftshift(M)); axis('square'); colormap(flipud(gray)); 
title('Masque de mesure dans plan de Fourier','FontSize',20);
disp('  -- paused'); pause;

figure(3); clf;
imagesc(log10(abs(fftshift(fft2(X))))); axis('square'); colormap(flipud(gray)); 
title('Log de Norme des coefficients de Fourier','FontSize',20);
xlabel('Axe variable mesuree','FontSize',20);
disp('  -- paused'); pause;

figure(3); clf;
imagesc(log10(abs(fftshift(M.*fft2(X))))); axis('square'); colormap(flipud(gray)); 
axis('square');
title('Log de Norme des coefficients de Fourier mesures','FontSize',20);
xlabel('Axe variable mesuree','FontSize',20);
disp('  -- paused'); pause;

figure(4); clf;
imagesc(Xbp); colormap(gray); axis('square')
title('Reconstruction en Optimisation l2','FontSize',20);
disp('  -- paused'); pause;

disp('---- Next Step : l1/TV recovery ----');

%% 2-b %% Recovery by l1-minimization of Total Variation

do_phantom = 1;

if do_phantom,              % a little bit too log -- Run beforehand
tveq_phantom_example;

% Recovery from l1 with equality constraints
xp = l1eq_pd(xbp, A, At, y, 1e-3, 70, 1e-8, 200);
Xl1eq = reshape(xp, n, n);

% Recovery from l1 with quadradtic constraints
epsilon1 = 5e-2;
xp2 = l1qc_logbarrier(xbp, A, At, y, epsilon1, 1e-3, 50, 1e-8, 500);
Xl1err = reshape(xp2, n, n);

else,
load matfile_tutorial

end;

%% figures correspondantes


figure(3); clf;
imagesc(Xl1eq); colormap(gray); axis('square')
title('Reconstruction en Optimisation L1 avec egalite','FontSize',20);
disp('  -- paused'); pause;

figure(3); clf;
imagesc(Xl1err); colormap(gray); axis('square')
title('Reconstruction en Optimisation L1 avec erreur','FontSize',20);
disp('  -- paused'); pause;

figure(2); clf;
imagesc(Xtv); colormap(gray); axis('square')
title('Reconstruction en Optimisation TV','FontSize',20);
disp('  -- paused'); pause;

disp('---- END of illustrations ----');

%%

disp('%%%%%% END OF TUTORIAL %%%%%%');



