% run_l1.m 
% Written by: Patrick Flandrin, Nelly Pustelnik and Pierre Borgnat, CNRS-ENS Lyon
% Email: {flandrin,nelly.pustelnik,pborgnat}@ens-lyon.fr
% Created: September 2007 - February 2015
% Modified: April 2015 (Nelly Pustelnik)
% ADAPTED FROM: 

% tveq_TFR_example.m
% Written by: Justin Romberg, Caltech
% Email: jrom@acm.caltech.edu
% Created: October 2005

% Comparison between L1prox and L1magic algorithmic solution

%% 1. creating the signal
clear all; close all; clc;
addpath(genpath('l1magic/'));
addpath 'include/'


%%
n = 256 ;
[s1,fs1] = fmsin(n,0.1,0.2,n/2) ; 
[s2,fs2] = fmlin(n,0.2,0.4) ;
a1 = amgauss(n,n/2,n/2) ; 
a2 = amgauss(n,n/2,n/2)/2 ;
s = a1.*s1 + a2.*s2 ; K = 1 ;
s = s/sqrt(sum(abs(s).^2)) ;

    
fs1d = min(n,1+round(2*n*fs1)) ; 
fs2d = min(n,1+round(2*n*fs2)) ; 
TF1 = zeros(n) ;
TF2 = zeros(n) ;
for kk = 1:n
    TF1(fs1d(kk),kk) = a1(kk).^2 ;
    TF2(fs2d(kk),kk) = a2(kk).^2 ;
end
TF = TF1 + TF2 ;
TF = TF/sum(sum(TF));


%% 2. Computing TF references

N = n ;
    

% (ressigned) spectrogram

[Xsp,Xrsp]=tfrrsp(s,1:N,2*N,tftb_window(105,'Gauss'),0);

% (smoothed) Wigner

X = tfrwv(s) ; % tfrspwv(s,1:n,n,tftb_window(9,'Gauss'),tftb_window(255,'Rect'),0) ; %tfrwv(s) ; %  
x = X(:) ;

% check plot

figure(100)

dyn = 20 ;

subplot(4,2,1); imageTF(TF,dyn,1,0); title('model')
subplot(4,2,2); imageTF(Xsp(1:N,:),dyn,1,0); title('spectro')
subplot(4,2,3); imageTF(X,dyn,1,0); title('WV')
subplot(4,2,4); imageTF(Xrsp(1:N,:),dyn,1,0); title('reass. spectro.')


%% 3. computing TF sparse solutions with l1_magic

larg = [12];

TF_l1prox = zeros(n,n,length(larg)) ;

%% loop for the different masks
adapt=1;
for k = 1:length(larg)
    for kk = 1%:length(larg)
        if adapt ==1
            % square mask in the central region of the ambiguity function
            M = zeros(N) ;
            d1 = larg(k) ;
            d2 = larg(k) ;	
            M(1+n/2-d1/2:1+n/2+d1/2,0+n/2-d2/2:0+n/2+d2/2) = ones(d1+1,d2+1) ;
            MK = M ;
        else
            % RGK
            alpha = 1 ;
            [tfr,Phj,sigma,its] = rgk(real(s),alpha) ;
            Phi = zeros(size(Phj)) ;
            Phi(:,1:N-1) = Phj(:,2:N) ;
            Phi_sort = sort(reshape(Phi,N*N,1)) ;
            seuil = Phi_sort(N^2-(larg+1)^2) ;
            M = Phi>=seuil ;
            MK = M ;
            M = fliplr(M) ;
        end

        proj=-1e10; 
        lambda=0.01;
        tic
        compute_prox; 
        t1 = toc;
        
        compute_l1magic; 
        subplot(4,2,5); imageTF(TF_l1prox(:,:,k),dyn,1,0); title('L1 prox')
        subplot(4,2,6); imageTF(TF_l1(:,:,k),dyn,1,0); title('l1 magic')
        subplot(4,2,7); imageTF(TF_l2(:,:,k),dyn,1,0); title('l2')
        disp('prox (time in s.):');disp(t1);
        disp('l1 magic  (time in s.) :');disp(t3);         
        disp('prox (EQM):');disp(norm(TF - TF_l1prox(:,:,k)/sum(sum(TF_l1prox(:,:,k))),'fro'))
        disp('l1 magic  (EQM) :'); disp(norm(TF - TF_l1(:,:,k)/sum(sum(TF_l1(:,:,k))),'fro'))
        disp('L2 (error)'); disp(norm(TF - TF_l2(:,:,k)/sum(sum(TF_l2(:,:,k))),'fro'))
        err(1,k) =  norm(TF - TF_l1prox(:,:,k)/sum(sum(TF_l1prox(:,:,k))),'fro');
        err(2,k) =  norm(TF - TF_l1(:,:,k)/sum(sum(TF_l1(:,:,k))),'fro');
        err(3,k) =  norm(TF - TF_l2(:,:,k)/sum(sum(TF_l2(:,:,k))),'fro'); 
        time(1,k) = t1;
        time(2,k) = t3;
        time(3,k) = t2;
     end
end

save('results_camsap_01.mat'); 
